AV = LibStub("AceAddon-3.0"):NewAddon("AutoVendor", "AceConsole-3.0", "AceEvent-3.0")

local options = {
  name = "AutoVendor",
  handler = AV,
  type = 'group',
  args = {
  	enable = {
  		type = 'toggle',
  		name = 'Enabled',
  		desc = 'Enable or disable this addon',
  		set = function(info, val) if (val) then AV:Enable() else AV:Disable() end end,
  		get = function(info) return AV.enabledState end,
  	},
  	empty = {
  		type = 'header',
  		cmdHidden = true,
  		name = "Options",
  	},
    verbosity = {
      type = 'select',
      name = 'Verbosity',
      desc = 'How much information is displayed when accessing a vendor',
      disabled = function() return not AV.enabledState end,
      values = {
      	none = 'None',
      	summary = 'Summary',
      	all = 'All',
      },
      set = 'SetVerbosity',
      get = 'GetVerbosity',
    },
  },
}

local defaults = {
	profile = {
		verbosity = 'summary',
		not_junk = {
			[1485] = "Pitchfork",
 		  [39202] = "Rusted Pitchfork",
		  [3944] = "Twill Belt",
		  [3945] = "Twill Boots",
		  [3946] = "Twill Bracers",
		  [3947] = "Twill Cloak",
		  [8754] = "Twill Cover",
		  [3948] = "Twill Gloves",
		  [3949] = "Twill Pants",
		  [3950] = "Twill Shoulderpads",
		  [3951] = "Twill Vest",
		  [18230] = "Broken I.W.I.N. Button",
	  },
	  junk = {
	  },
	}
}

function AV:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AutoVendorDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AutoVendor", options, {"autovendor", "av"})
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AutoVendor", "AutoVendor")
end

function AV:OnEnable()
  self:RegisterEvent("MERCHANT_SHOW")
  self:RegisterChatCommand("junklist", "JunkList")
  self:RegisterChatCommand("notjunklist", "NotJunkList")
  self:RegisterChatCommand("junk", "ToggleJunk")
  self:RegisterChatCommand("notjunk", "ToggleNotJunk")
end

function AV:OnDisable()
end

function AV:SetVerbosity(info, val)
	self.db.profile.verbosity = val
	self:Print("Setting verbosity level to '"..val.."'")
end

function AV:GetVerbosity(info)
	return self.db.profile.verbosity
end

local function listFormat(list)
	local tmpList = {}
	for _,v in pairs(list) do
		table.insert(tmpList, v)
	end
	table.sort(tmpList)
	tmpString = ''
	for k,v in pairs(list) do
		local item_link = select(2, GetItemInfo(k))
		if item_link == nil then
			item_link = v
		end
		if #tmpString > 0 and #tmpString + #item_link <= 255 then
			tmpString = tmpString .. ', ' .. item_link
		else
			if #tmpString == 0 then
				tmpString = item_link
			else
				AV:Print(tmpString)
				tmpString = item_link
			end
		end
	end
	if #tmpString ~= 0 then
		AV:Print(tmpString)
	end
end

local function listRemove(list, item)
	found = false
	for k,v in pairs(list) do
		if string.lower(v) == string.lower(item) then
			list[k] = nil
			found = true
		end
	end
	return found
end

local function listToggle(list, listName, itemId, itemName)
	if list[itemId] then
		list[itemId] = nil
		AV:Print('Removed '..itemName..' from '..listName..'.')
	else
		table.insert(list, itemId, itemName)
		AV:Print('Added '..itemName..' to '..listName..'.')
	end
end

function AV:ToggleJunk(msg, editbox)
	if msg then
		local itemId = tonumber(strmatch(msg, "item:(%d+)"))
		local itemName = select(1, GetItemInfo(msg))
		if itemId and itemName then
			listToggle(self.db.profile.junk, 'junk list', itemId, itemName)
		else
			if msg and listRemove(self.db.profile.junk, msg) then
				self:Print('Removed '..msg..' from junk list.')
			else
				self:Print('No item (link) supplied!')
			end
		end
	else
		self:Print('No item link supplied!')
	end
end

function AV:ToggleNotJunk(msg, editbox)
	if msg then
		local itemId = tonumber(strmatch(msg, "item:(%d+)"))
		local itemName = select(1, GetItemInfo(msg))
		if itemId and itemName then
			listToggle(self.db.profile.not_junk, 'not junk list', itemId, itemName)
		else
			if msg and listRemove(self.db.profile.not_junk, msg) then
				self:Print('Removed '..msg..' from not junk list.')
			else
				self:Print('No item (link) supplied!')
			end
		end
	else
		self:Print('No item (link) supplied!')
	end
end

function AV:JunkList(msg, editbox)
	local empty = true
	for _,_ in pairs(self.db.profile.junk) do
		empty = false
	end
	if empty then
		self:Print('The junk list is empty.')
	else
		self:Print('Items in the junk list:')
	end
	listFormat(self.db.profile.junk)
end

function AV:NotJunkList(msg, editbox)
	local empty = true
	for _,_ in pairs(self.db.profile.not_junk) do
		empty = false
	end
	if empty then
		self:Print('The not-junk list is empty.')
	else
		self:Print('Items in the not junk list:')
	end
	listFormat(self.db.profile.not_junk)
end

function AV:MERCHANT_SHOW()
	local iconSize = select(2, GetChatWindowInfo(1)) - 2
	local totalSellValue = 0
	local totalItemsSold = 0
	for bag=0,4 do
		for slot=0,GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if link then
				local itemId = tonumber(strmatch(link, "item:(%d+)"))
				if (select(3, GetItemInfo(link)) == 0 or self.db.profile.junk[itemId]) and not self.db.profile.not_junk[itemId] and itemId ~= 6196 then
					ShowMerchantSellCursor(1)
					UseContainerItem(bag, slot)
					local itemCount = select(2, GetContainerItemInfo(bag, slot))
					local sellValue = itemCount * select(11, GetItemInfo(link))
					totalSellValue = totalSellValue + sellValue
					totalItemsSold = totalItemsSold + 1
					if self.db.profile.verbosity == 'all' then
						self:Print(format("Selling %sx%d for %s.", link, itemCount, GetCoinTextureString(sellValue, iconSize)))
					end
				end
			end
		end
	end
	if totalItemsSold > 0 and (self.db.profile.verbosity == 'all' or self.db.profile.verbosity == 'summary') then
		local items = 'items'
		if totalItemsSold == 1 then
			items = 'item'
		end
		self:Print(format("Automatically sold %d %s for %s.", totalItemsSold, items, GetCoinTextureString(totalSellValue, iconSize)))
	end
end
